# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._payment_location_capability_list_params import (
        PaymentLocationCapabilityListParams,
    )
    from stripe.params._payment_location_capability_modify_params import (
        PaymentLocationCapabilityModifyParams,
    )
    from stripe.params._payment_location_capability_retrieve_params import (
        PaymentLocationCapabilityRetrieveParams,
    )


class PaymentLocationCapability(
    ListableAPIResource["PaymentLocationCapability"],
    UpdateableAPIResource["PaymentLocationCapability"],
):
    """
    A `payment_location` capability represents a capability for a Stripe account at a payment location.
    """

    OBJECT_NAME: ClassVar[Literal["payment_location_capability"]] = (
        "payment_location_capability"
    )

    class Requirements(StripeObject):
        class Error(StripeObject):
            code: Literal["information_missing", "invalid_value_other"]
            """
            The code for the type of error.
            """
            reason: str
            """
            An informative message that indicates the error type and provides additional details about the error.
            """
            requirement: str
            """
            The specific user onboarding requirement field (in the requirements hash) that needs to be resolved.
            """

        currently_due: List[str]
        """
        Fields that need to be collected to keep the capability enabled.
        """
        disabled_reason: Optional[
            Literal[
                "account.capability_required",
                "pending.onboarding",
                "pending.review",
                "rejected.other",
                "rejected.unsupported_business",
                "requirements.fields_needed",
            ]
        ]
        """
        Description of why the capability is disabled.
        """
        errors: List[Error]
        """
        Fields that are `currently_due` and need to be collected again because validation or verification failed.
        """
        _inner_class_types = {"errors": Error}

    account: str
    """
    The account that the capability enables functionality for.
    """
    capability: Literal["fr_meal_vouchers_conecs_payments"]
    """
    The identifier for the capability.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    location: str
    """
    The payment location that the capability enables functionality for.
    """
    object: Literal["payment_location_capability"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    requested: bool
    """
    Whether the capability has been requested.
    """
    requested_at: Optional[int]
    """
    Time when the capability was requested. Measured in seconds since the Unix epoch.
    """
    requirements: Requirements
    status: Literal["active", "inactive", "pending", "unrequested"]
    """
    The status of the capability.
    """

    @classmethod
    def list(
        cls, **params: Unpack["PaymentLocationCapabilityListParams"]
    ) -> ListObject["PaymentLocationCapability"]:
        """
        List all payment location capabilities associated with the payment location.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["PaymentLocationCapabilityListParams"]
    ) -> ListObject["PaymentLocationCapability"]:
        """
        List all payment location capabilities associated with the payment location.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["PaymentLocationCapabilityModifyParams"]
    ) -> "PaymentLocationCapability":
        """
        Updates a payment_location capability. Request or remove a payment_location capability by updating its requested parameter.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentLocationCapability",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["PaymentLocationCapabilityModifyParams"]
    ) -> "PaymentLocationCapability":
        """
        Updates a payment_location capability. Request or remove a payment_location capability by updating its requested parameter.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentLocationCapability",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls,
        id: str,
        **params: Unpack["PaymentLocationCapabilityRetrieveParams"],
    ) -> "PaymentLocationCapability":
        """
        Retrieves a payment_location capability
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls,
        id: str,
        **params: Unpack["PaymentLocationCapabilityRetrieveParams"],
    ) -> "PaymentLocationCapability":
        """
        Retrieves a payment_location capability
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def class_url(cls):
        return "/v1/payment_location_capabilities"

    _inner_class_types = {"requirements": Requirements}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class LocationCreateParams(RequestOptions):
    address: "LocationCreateParamsAddress"
    """
    The physical address of the tax location.
    """
    description: NotRequired[str]
    """
    Details to identify the tax location by its venue, types of events held, or available services, such as "A spacious auditorium suitable for large concerts and events.".
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    type: Literal["performance"]
    """
    The type of tax location. The only supported value is "performance".
    """


class LocationCreateParamsAddress(TypedDict):
    city: NotRequired["Literal['']|str"]
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired["Literal['']|str"]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired["Literal['']|str"]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired["Literal['']|str"]
    """
    ZIP or postal code.
    """
    state: NotRequired["Literal['']|str"]
    """
    State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class OrderEvent(StripeObject):
    """
    An order event represents a change to a delegated checkout order.
    """

    OBJECT_NAME: ClassVar[Literal["delegated_checkout.order_event"]] = (
        "delegated_checkout.order_event"
    )

    class Adjustment(StripeObject):
        class LineItem(StripeObject):
            key: str
            """
            The line item key.
            """
            quantity: int
            """
            The quantity associated with the order event.
            """

        amount: Optional[int]
        """
        The amount associated with the adjustment.
        """
        currency: Optional[str]
        """
        The currency associated with the adjustment amount.
        """
        description: str
        """
        The description of the adjustment.
        """
        line_items: Optional[List[LineItem]]
        """
        The line items associated with the adjustment.
        """
        status: Literal["completed", "failed", "pending"]
        """
        The status of the adjustment.
        """
        type: Literal[
            "cancellation",
            "credit",
            "dispute",
            "original_payment_refund",
            "return",
            "store_credit_refund",
        ]
        """
        The type of adjustment.
        """
        _inner_class_types = {"line_items": LineItem}

    class Fulfillment(StripeObject):
        class LineItem(StripeObject):
            key: str
            """
            The line item key.
            """
            quantity: int
            """
            The quantity associated with the order event.
            """

        carrier: Optional[str]
        """
        The carrier for the fulfillment.
        """
        delivered_at: Optional[int]
        """
        Time at which the fulfillment was delivered. Measured in seconds since the Unix epoch.
        """
        line_items: List[LineItem]
        """
        The line items associated with the fulfillment.
        """
        shipped_at: Optional[int]
        """
        Time at which the fulfillment shipped. Measured in seconds since the Unix epoch.
        """
        status: Literal[
            "confirmed",
            "delivered",
            "fulfilled",
            "pending",
            "returned",
            "shipped",
        ]
        """
        The status of the fulfillment.
        """
        tracking_number: Optional[str]
        """
        The tracking number for the fulfillment.
        """
        tracking_url: Optional[str]
        """
        The tracking URL for the fulfillment.
        """
        _inner_class_types = {"line_items": LineItem}

    adjustment: Optional[Adjustment]
    """
    The adjustment details for this order event.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    fulfillment: Optional[Fulfillment]
    """
    The fulfillment details for this order event.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["delegated_checkout.order_event"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    occurred_at: int
    """
    Time at which this event occurred. Measured in seconds since the Unix epoch.
    """
    order: str
    """
    The delegated checkout order associated with this order event.
    """
    requested_session: str
    """
    The requested session associated with this order event.
    """
    type: Literal["adjustment", "fulfillment"]
    """
    The type of order event.
    """
    _inner_class_types = {"adjustment": Adjustment, "fulfillment": Fulfillment}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._nested_resource_class_methods import nested_resource_class_methods
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._account import Account
    from stripe._application import Application
    from stripe._application_fee_refund import ApplicationFeeRefund
    from stripe._balance_transaction import BalanceTransaction
    from stripe._charge import Charge
    from stripe.params._application_fee_create_refund_params import (
        ApplicationFeeCreateRefundParams,
    )
    from stripe.params._application_fee_list_params import (
        ApplicationFeeListParams,
    )
    from stripe.params._application_fee_list_refunds_params import (
        ApplicationFeeListRefundsParams,
    )
    from stripe.params._application_fee_modify_refund_params import (
        ApplicationFeeModifyRefundParams,
    )
    from stripe.params._application_fee_refund_params import (
        ApplicationFeeRefundParams,
    )
    from stripe.params._application_fee_retrieve_params import (
        ApplicationFeeRetrieveParams,
    )
    from stripe.params._application_fee_retrieve_refund_params import (
        ApplicationFeeRetrieveRefundParams,
    )


@nested_resource_class_methods("refund")
class ApplicationFee(ListableAPIResource["ApplicationFee"]):
    OBJECT_NAME: ClassVar[Literal["application_fee"]] = "application_fee"

    class FeeSource(StripeObject):
        charge: Optional[str]
        """
        Charge ID that created this application fee.
        """
        payment_record: Optional[str]
        """
        PaymentRecord ID that created this application fee.
        """
        payout: Optional[str]
        """
        Payout ID that created this application fee.
        """
        transfer: Optional[str]
        """
        Transfer ID that created this application fee.
        """
        type: Literal["charge", "payout", "transfer"]
        """
        Type of object that created the application fee.
        """

    class FundingSource(StripeObject):
        invoice: Optional[str]
        """
        The invoice ID associated with this funding source, if applicable.
        """
        type: str
        """
        The type of funding source.
        """

    account: ExpandableField["Account"]
    """
    ID of the Stripe account this fee was taken from.
    """
    amount: int
    """
    Amount earned, in cents (or local equivalent).
    """
    amount_refunded: int
    """
    Amount in cents (or local equivalent) refunded (can be less than the amount attribute on the fee if a partial refund was issued)
    """
    application: ExpandableField["Application"]
    """
    ID of the Connect application that earned the fee.
    """
    balance_transaction: Optional[ExpandableField["BalanceTransaction"]]
    """
    Balance transaction that describes the impact of this collected application fee on your account balance (not including refunds).
    """
    charge: ExpandableField["Charge"]
    """
    ID of the charge that the application fee was taken from.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    fee_source: Optional[FeeSource]
    """
    Polymorphic source of the application fee. Includes the ID of the object the application fee was created from.
    """
    funding_source: Optional[FundingSource]
    """
    Polymorphic funding source of the application fee. Includes the type and details of the funding source.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["application_fee"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    originating_transaction: Optional[ExpandableField["Charge"]]
    """
    ID of the corresponding charge on the platform account, if this fee was the result of a charge using the `destination` parameter.
    """
    refunded: bool
    """
    Whether the fee has been fully refunded. If the fee is only partially refunded, this attribute will still be false.
    """
    refunds: ListObject["ApplicationFeeRefund"]
    """
    A list of refunds that have been applied to the fee.
    """
    settlement_type: Optional[Literal["gross_settled", "net_settled"]]
    """
    Type of settlement for the application fee. One of `net_settled` or `gross_settled`.
    """

    @classmethod
    def list(
        cls, **params: Unpack["ApplicationFeeListParams"]
    ) -> ListObject["ApplicationFee"]:
        """
        Returns a list of application fees you've previously collected. The application fees are returned in sorted order, with the most recent fees appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ApplicationFeeListParams"]
    ) -> ListObject["ApplicationFee"]:
        """
        Returns a list of application fees you've previously collected. The application fees are returned in sorted order, with the most recent fees appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def _cls_refund(
        cls, id: str, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            cls._static_request(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def refund(
        id: str, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        ...

    @overload
    def refund(
        self, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        ...

    @class_method_variant("_cls_refund")
    def refund(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            self._request(
                "post",
                "/v1/application_fees/{id}/refunds".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_refund_async(
        cls, id: str, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await cls._static_request_async(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def refund_async(
        id: str, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        ...

    @overload
    async def refund_async(
        self, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        ...

    @class_method_variant("_cls_refund_async")
    async def refund_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["ApplicationFeeRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await self._request_async(
                "post",
                "/v1/application_fees/{id}/refunds".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ApplicationFeeRetrieveParams"]
    ) -> "ApplicationFee":
        """
        Retrieves the details of an application fee that your account has collected. The same information is returned when refunding the application fee.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ApplicationFeeRetrieveParams"]
    ) -> "ApplicationFee":
        """
        Retrieves the details of an application fee that your account has collected. The same information is returned when refunding the application fee.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def retrieve_refund(
        cls,
        fee: str,
        id: str,
        **params: Unpack["ApplicationFeeRetrieveRefundParams"],
    ) -> "ApplicationFeeRefund":
        """
        By default, you can see the 10 most recent refunds stored directly on the application fee object, but you can also retrieve details about a specific refund stored on the application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            cls._static_request(
                "get",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def retrieve_refund_async(
        cls,
        fee: str,
        id: str,
        **params: Unpack["ApplicationFeeRetrieveRefundParams"],
    ) -> "ApplicationFeeRefund":
        """
        By default, you can see the 10 most recent refunds stored directly on the application fee object, but you can also retrieve details about a specific refund stored on the application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await cls._static_request_async(
                "get",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    def modify_refund(
        cls,
        fee: str,
        id: str,
        **params: Unpack["ApplicationFeeModifyRefundParams"],
    ) -> "ApplicationFeeRefund":
        """
        Updates the specified application fee refund by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata as an argument.
        """
        return cast(
            "ApplicationFeeRefund",
            cls._static_request(
                "post",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def modify_refund_async(
        cls,
        fee: str,
        id: str,
        **params: Unpack["ApplicationFeeModifyRefundParams"],
    ) -> "ApplicationFeeRefund":
        """
        Updates the specified application fee refund by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata as an argument.
        """
        return cast(
            "ApplicationFeeRefund",
            await cls._static_request_async(
                "post",
                "/v1/application_fees/{fee}/refunds/{id}".format(
                    fee=sanitize_id(fee), id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    def list_refunds(
        cls, id: str, **params: Unpack["ApplicationFeeListRefundsParams"]
    ) -> ListObject["ApplicationFeeRefund"]:
        """
        You can see a list of the refunds belonging to a specific application fee. Note that the 10 most recent refunds are always available by default on the application fee object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional refunds.
        """
        return cast(
            ListObject["ApplicationFeeRefund"],
            cls._static_request(
                "get",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @classmethod
    async def list_refunds_async(
        cls, id: str, **params: Unpack["ApplicationFeeListRefundsParams"]
    ) -> ListObject["ApplicationFeeRefund"]:
        """
        You can see a list of the refunds belonging to a specific application fee. Note that the 10 most recent refunds are always available by default on the application fee object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional refunds.
        """
        return cast(
            ListObject["ApplicationFeeRefund"],
            await cls._static_request_async(
                "get",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @classmethod
    def create_refund(
        cls, id: str, **params: Unpack["ApplicationFeeCreateRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            cls._static_request(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @classmethod
    async def create_refund_async(
        cls, id: str, **params: Unpack["ApplicationFeeCreateRefundParams"]
    ) -> "ApplicationFeeRefund":
        """
        Refunds an application fee that has previously been collected but not yet refunded.
        Funds will be refunded to the Stripe account from which the fee was originally collected.

        You can optionally refund only part of an application fee.
        You can do so multiple times, until the entire fee has been refunded.

        Once entirely refunded, an application fee can't be refunded again.
        This method will raise an error when called on an already-refunded application fee,
        or when trying to refund more money than is left on an application fee.
        """
        return cast(
            "ApplicationFeeRefund",
            await cls._static_request_async(
                "post",
                "/v1/application_fees/{id}/refunds".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    _inner_class_types = {
        "fee_source": FeeSource,
        "funding_source": FundingSource,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class RequestedSessionModifyParams(RequestOptions):
    discounts: NotRequired["RequestedSessionModifyParamsDiscounts"]
    """
    The discount codes to apply to this requested session. Pass an empty value to remove all applied discounts.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fulfillment_details: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetails"
    ]
    """
    The details of the fulfillment.
    """
    line_item_details: NotRequired[
        List["RequestedSessionModifyParamsLineItemDetail"]
    ]
    """
    The details of the line items.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    The metadata for this requested session.
    """
    payment_method: NotRequired[str]
    """
    The payment method for this requested session.
    """
    payment_method_options: NotRequired[
        "RequestedSessionModifyParamsPaymentMethodOptions"
    ]
    """
    The payment method options for this requested session.
    """
    shared_metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    The shared metadata for this requested session.
    """


class RequestedSessionModifyParamsDiscounts(TypedDict):
    codes: Union[Literal[""], List[str]]
    """
    Array of discount codes to apply. Pass an empty value to remove all applied discounts.
    """
    enforce_strict_eligibility: NotRequired[bool]
    """
    Whether to enforce strict eligibility for discount codes. Defaults to true. When false, invalid codes are returned in the discounts.invalid array instead of raising an error.
    """


class RequestedSessionModifyParamsFulfillmentDetails(TypedDict):
    address: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsAddress"
    ]
    """
    The customer's address.
    """
    email: NotRequired[str]
    """
    The customer's email address.
    """
    name: NotRequired[str]
    """
    The customer's name.
    """
    phone: NotRequired[str]
    """
    The customer's phone number.
    """
    selected_fulfillment_option: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOption"
    ]
    """
    The fulfillment option to select.
    """
    selected_fulfillment_option_overrides: NotRequired[
        List[
            "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverride"
        ]
    ]
    """
    The fulfillment option overrides for specific line items.
    """


class RequestedSessionModifyParamsFulfillmentDetailsAddress(TypedDict):
    city: str
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: str
    """
    ZIP or postal code.
    """
    state: str
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOption(
    TypedDict,
):
    digital: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionDigital"
    ]
    """
    The digital fulfillment option.
    """
    shipping: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionShipping"
    ]
    """
    The shipping fulfillment option.
    """
    type: Literal["digital", "shipping"]
    """
    The type of fulfillment option.
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionDigital(
    TypedDict,
):
    digital_option: str
    """
    The digital option identifier.
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionShipping(
    TypedDict,
):
    shipping_option: str
    """
    The shipping option identifier.
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverride(
    TypedDict,
):
    digital: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital"
    ]
    """
    The digital fulfillment option.
    """
    line_item_keys: List[str]
    """
    The line item keys that this fulfillment option override applies to.
    """
    shipping: NotRequired[
        "RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping"
    ]
    """
    The shipping fulfillment option.
    """
    type: Literal["digital", "shipping"]
    """
    The type of fulfillment option.
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideDigital(
    TypedDict,
):
    digital_option: str
    """
    The digital option identifier.
    """


class RequestedSessionModifyParamsFulfillmentDetailsSelectedFulfillmentOptionOverrideShipping(
    TypedDict,
):
    shipping_option: str
    """
    The shipping option identifier.
    """


class RequestedSessionModifyParamsLineItemDetail(TypedDict):
    key: str
    """
    The key of the line item.
    """
    quantity: int
    """
    The quantity of the line item.
    """


class RequestedSessionModifyParamsPaymentMethodOptions(TypedDict):
    card: NotRequired["RequestedSessionModifyParamsPaymentMethodOptionsCard"]
    """
    Card-specific payment method options.
    """
    excluded_payment_method_types: NotRequired[
        List[Literal["affirm", "card", "klarna"]]
    ]
    """
    The payment method types to exclude from the session.
    """


class RequestedSessionModifyParamsPaymentMethodOptionsCard(TypedDict):
    brands_blocked: NotRequired[
        List[Literal["american_express", "mastercard", "visa"]]
    ]
    """
    The card brands to exclude from the session.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class OneTimeItem(StripeObject):
    """
    A one-time item represents a product that can be charged as a one-time line item,
    used for overage charges when custom pricing unit credits are exhausted.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.one_time_item"]] = (
        "v2.billing.one_time_item"
    )

    class TaxDetails(StripeObject):
        tax_code: str
        """
        Product tax code (PTC).
        """

    created: str
    """
    Timestamp of when the object was created.
    """
    display_name: str
    """
    Description that customers see in the invoice line item.
    Maximum length of 250 characters.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: Optional[str]
    """
    An internal key you can use to search for a particular one-time item.
    Maximum length of 200 characters.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.one_time_item"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    tax_details: Optional[TaxDetails]
    """
    Stripe Tax details.
    """
    unit_label: Optional[str]
    """
    The unit to use when displaying prices for this one-time item. For example, set this field
    to "credit" for the display to show "(price) per credit".
    Maximum length of 100 characters.
    """
    _inner_class_types = {"tax_details": TaxDetails}



# -*- coding: utf-8 -*-
# File copied from our code generator; changes here will be overwritten.
# -*- coding: utf-8 -*-

from typing_extensions import TYPE_CHECKING
from stripe.v2.core._event import (
    EventNotification as EventNotification,
    RelatedObject as RelatedObject,
    Reason as Reason,
    ReasonRequest as ReasonRequest,
)

# The beginning of the section generated from our OpenAPI spec
from importlib import import_module

if TYPE_CHECKING:
    from stripe.v2.core import (
        accounts as accounts,
        health as health,
        vault as vault,
    )
    from stripe.v2.core._account import Account as Account
    from stripe.v2.core._account_evaluation import (
        AccountEvaluation as AccountEvaluation,
    )
    from stripe.v2.core._account_evaluation_service import (
        AccountEvaluationService as AccountEvaluationService,
    )
    from stripe.v2.core._account_link import AccountLink as AccountLink
    from stripe.v2.core._account_link_service import (
        AccountLinkService as AccountLinkService,
    )
    from stripe.v2.core._account_person import AccountPerson as AccountPerson
    from stripe.v2.core._account_person_token import (
        AccountPersonToken as AccountPersonToken,
    )
    from stripe.v2.core._account_service import (
        AccountService as AccountService,
    )
    from stripe.v2.core._account_token import AccountToken as AccountToken
    from stripe.v2.core._account_token_service import (
        AccountTokenService as AccountTokenService,
    )
    from stripe.v2.core._approval_request import (
        ApprovalRequest as ApprovalRequest,
    )
    from stripe.v2.core._approval_request_service import (
        ApprovalRequestService as ApprovalRequestService,
    )
    from stripe.v2.core._batch_job import BatchJob as BatchJob
    from stripe.v2.core._batch_job_service import (
        BatchJobService as BatchJobService,
    )
    from stripe.v2.core._claimable_sandbox import (
        ClaimableSandbox as ClaimableSandbox,
    )
    from stripe.v2.core._claimable_sandbox_service import (
        ClaimableSandboxService as ClaimableSandboxService,
    )
    from stripe.v2.core._connection_session import (
        ConnectionSession as ConnectionSession,
    )
    from stripe.v2.core._connection_session_service import (
        ConnectionSessionService as ConnectionSessionService,
    )
    from stripe.v2.core._event import Event as Event
    from stripe.v2.core._event_destination import (
        EventDestination as EventDestination,
    )
    from stripe.v2.core._event_destination_service import (
        EventDestinationService as EventDestinationService,
    )
    from stripe.v2.core._event_service import EventService as EventService
    from stripe.v2.core._fee_batch import FeeBatch as FeeBatch
    from stripe.v2.core._fee_batch_service import (
        FeeBatchService as FeeBatchService,
    )
    from stripe.v2.core._fee_entry import FeeEntry as FeeEntry
    from stripe.v2.core._fee_entry_service import (
        FeeEntryService as FeeEntryService,
    )
    from stripe.v2.core._health_service import HealthService as HealthService
    from stripe.v2.core._vault_service import VaultService as VaultService

# name -> (import_target, is_submodule)
_import_map = {
    "accounts": ("stripe.v2.core.accounts", True),
    "health": ("stripe.v2.core.health", True),
    "vault": ("stripe.v2.core.vault", True),
    "Account": ("stripe.v2.core._account", False),
    "AccountEvaluation": ("stripe.v2.core._account_evaluation", False),
    "AccountEvaluationService": (
        "stripe.v2.core._account_evaluation_service",
        False,
    ),
    "AccountLink": ("stripe.v2.core._account_link", False),
    "AccountLinkService": ("stripe.v2.core._account_link_service", False),
    "AccountPerson": ("stripe.v2.core._account_person", False),
    "AccountPersonToken": ("stripe.v2.core._account_person_token", False),
    "AccountService": ("stripe.v2.core._account_service", False),
    "AccountToken": ("stripe.v2.core._account_token", False),
    "AccountTokenService": ("stripe.v2.core._account_token_service", False),
    "ApprovalRequest": ("stripe.v2.core._approval_request", False),
    "ApprovalRequestService": (
        "stripe.v2.core._approval_request_service",
        False,
    ),
    "BatchJob": ("stripe.v2.core._batch_job", False),
    "BatchJobService": ("stripe.v2.core._batch_job_service", False),
    "ClaimableSandbox": ("stripe.v2.core._claimable_sandbox", False),
    "ClaimableSandboxService": (
        "stripe.v2.core._claimable_sandbox_service",
        False,
    ),
    "ConnectionSession": ("stripe.v2.core._connection_session", False),
    "ConnectionSessionService": (
        "stripe.v2.core._connection_session_service",
        False,
    ),
    "Event": ("stripe.v2.core._event", False),
    "EventDestination": ("stripe.v2.core._event_destination", False),
    "EventDestinationService": (
        "stripe.v2.core._event_destination_service",
        False,
    ),
    "EventService": ("stripe.v2.core._event_service", False),
    "FeeBatch": ("stripe.v2.core._fee_batch", False),
    "FeeBatchService": ("stripe.v2.core._fee_batch_service", False),
    "FeeEntry": ("stripe.v2.core._fee_entry", False),
    "FeeEntryService": ("stripe.v2.core._fee_entry_service", False),
    "HealthService": ("stripe.v2.core._health_service", False),
    "VaultService": ("stripe.v2.core._vault_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()

# The end of the section generated from our OpenAPI spec



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ApprovalRequestSubmitParams(TypedDict):
    reason: NotRequired[str]
    """
    The reason for submitting the approval request.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ApiKeyListParams(TypedDict):
    include_expired: NotRequired[bool]
    """
    Whether to include expired API keys in the response.
    """
    limit: NotRequired[int]
    """
    Limit of items to return per page.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._fr_meal_vouchers_onboarding_create_params import (
        FrMealVouchersOnboardingCreateParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_list_params import (
        FrMealVouchersOnboardingListParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_modify_params import (
        FrMealVouchersOnboardingModifyParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_retrieve_params import (
        FrMealVouchersOnboardingRetrieveParams,
    )


class FrMealVouchersOnboarding(
    CreateableAPIResource["FrMealVouchersOnboarding"],
    ListableAPIResource["FrMealVouchersOnboarding"],
    UpdateableAPIResource["FrMealVouchersOnboarding"],
):
    """
    The `French Meal Vouchers Onboarding` resource encapsulates the onboarding status and other related information
    for a single restaurant (SIRET number) in the context of the French Meal Vouchers program.

    To onboard a restaurant for the French Meal Vouchers program, you create a `French Meal Vouchers Onboarding` object.
    You can retrieve individual objects, list all such objects, or update objects to correct the postal code of the restaurant.
    We identify `French Meal Vouchers Onboarding` objects with a unique, random ID.

    Related guide: [Set up a restaurant for titres-restaurant payments](https://docs.stripe.com/payments/meal-vouchers/fr-meal-vouchers/set-up-restaurant)
    """

    OBJECT_NAME: ClassVar[Literal["fr_meal_vouchers_onboarding"]] = (
        "fr_meal_vouchers_onboarding"
    )

    class Providers(StripeObject):
        class Conecs(StripeObject):
            class Issuers(StripeObject):
                available: List[Literal["bimpli", "edenred", "pluxee", "up"]]
                """
                Issuers are available to this restaurant via Conecs, will be blank if the onboarding to Conecs is not complete or unsuccessful
                """

            class Requirements(StripeObject):
                class Error(StripeObject):
                    code: Literal["postal_code_invalid", "siret_invalid"]
                    """
                    The code for the type of error.
                    """
                    message: str
                    """
                    An informative message that provides additional details about the error.
                    """
                    requirement: Optional[Literal["postal_code", "siret"]]
                    """
                    The specific onboarding requirement field (in the requirements hash) that needs to be resolved.
                    """

                errors: List[Error]
                """
                Information any errors that are preventing the onboarding to Conecs from being completed.
                """
                past_due: List[Literal["postal_code", "siret"]]
                """
                Fields that need to be provided to complete the onboarding to Conecs.
                """
                _inner_class_types = {"errors": Error}

            issuers: Issuers
            """
            This represents information which issuers are available to this restaurant via Conecs
            """
            requirements: Requirements
            """
            This represents information about outstanding requirements for this restaurant to onboard to Conecs
            """
            status: Literal[
                "action_required", "active", "disentitled", "pending"
            ]
            """
            Status of the restaurant's onboarding to Conecs
            """
            _inner_class_types = {
                "issuers": Issuers,
                "requirements": Requirements,
            }

        conecs: Conecs
        """
        This represents the onboarding state of the restaurant on Conecs.
        """
        _inner_class_types = {"conecs": Conecs}

    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: str
    """
    Name of the restaurant.
    """
    object: Literal["fr_meal_vouchers_onboarding"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    postal_code: str
    """
    Postal code of the restaurant.
    """
    providers: Providers
    """
    This represents the onboarding state of the restaurant on different providers.
    """
    siret: str
    """
    SIRET number associated with the restaurant.
    """

    @classmethod
    def create(
        cls, **params: Unpack["FrMealVouchersOnboardingCreateParams"]
    ) -> "FrMealVouchersOnboarding":
        """
        Creates a French Meal Vouchers Onboarding object that represents a restaurant's onboarding status and starts the onboarding process.
        """
        return cast(
            "FrMealVouchersOnboarding",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["FrMealVouchersOnboardingCreateParams"]
    ) -> "FrMealVouchersOnboarding":
        """
        Creates a French Meal Vouchers Onboarding object that represents a restaurant's onboarding status and starts the onboarding process.
        """
        return cast(
            "FrMealVouchersOnboarding",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["FrMealVouchersOnboardingListParams"]
    ) -> ListObject["FrMealVouchersOnboarding"]:
        """
        Lists French Meal Vouchers Onboarding objects. The objects are returned in sorted order, with the most recently created objects appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["FrMealVouchersOnboardingListParams"]
    ) -> ListObject["FrMealVouchersOnboarding"]:
        """
        Lists French Meal Vouchers Onboarding objects. The objects are returned in sorted order, with the most recently created objects appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["FrMealVouchersOnboardingModifyParams"]
    ) -> "FrMealVouchersOnboarding":
        """
        Updates the details of a restaurant's French Meal Vouchers Onboarding object by
        setting the values of the parameters passed. Any parameters not provided are left unchanged.
        After you update the object, the onboarding process automatically restarts.

        You can only update French Meal Vouchers Onboarding objects with the postal_code field requirement in past_due.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "FrMealVouchersOnboarding",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["FrMealVouchersOnboardingModifyParams"]
    ) -> "FrMealVouchersOnboarding":
        """
        Updates the details of a restaurant's French Meal Vouchers Onboarding object by
        setting the values of the parameters passed. Any parameters not provided are left unchanged.
        After you update the object, the onboarding process automatically restarts.

        You can only update French Meal Vouchers Onboarding objects with the postal_code field requirement in past_due.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "FrMealVouchersOnboarding",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls,
        id: str,
        **params: Unpack["FrMealVouchersOnboardingRetrieveParams"],
    ) -> "FrMealVouchersOnboarding":
        """
        Retrieves the details of a previously created French Meal Vouchers Onboarding object.

        Supply the unique French Meal Vouchers Onboarding ID that was returned from your previous request,
        and Stripe returns the corresponding onboarding information.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls,
        id: str,
        **params: Unpack["FrMealVouchersOnboardingRetrieveParams"],
    ) -> "FrMealVouchersOnboarding":
        """
        Retrieves the details of a previously created French Meal Vouchers Onboarding object.

        Supply the unique French Meal Vouchers Onboarding ID that was returned from your previous request,
        and Stripe returns the corresponding onboarding information.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"providers": Providers}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class SetupIntentConfirmParams(RequestOptions):
    confirmation_token: NotRequired[str]
    """
    ID of the ConfirmationToken used to confirm this SetupIntent.

    If the provided ConfirmationToken contains properties that are also being provided in this request, such as `payment_method`, then the values in this request will take precedence.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    mandate_data: NotRequired[
        "Literal['']|SetupIntentConfirmParamsMandateData"
    ]
    payment_method: NotRequired[str]
    """
    ID of the payment method (a PaymentMethod, Card, or saved Source object) to attach to this SetupIntent.
    """
    payment_method_data: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodData"
    ]
    """
    When included, this hash creates a PaymentMethod that is set as the [`payment_method`](https://docs.stripe.com/api/setup_intents/object#setup_intent_object-payment_method)
    value in the SetupIntent.
    """
    payment_method_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptions"
    ]
    """
    Payment method-specific configuration for this SetupIntent.
    """
    return_url: NotRequired[str]
    """
    The URL to redirect your customer back to after they authenticate on the payment method's app or site.
    If you'd prefer to redirect to a mobile application, you can alternatively supply an application URI scheme.
    This parameter is only used for cards and other redirect-based payment methods.
    """
    setup_details: NotRequired["SetupIntentConfirmParamsSetupDetails"]
    """
    Provides industry-specific information about the SetupIntent.
    """
    use_stripe_sdk: NotRequired[bool]
    """
    Set to `true` when confirming server-side and using Stripe.js, iOS, or Android client-side SDKs to handle the next actions.
    """


class SetupIntentConfirmParamsMandateData(TypedDict):
    customer_acceptance: NotRequired[
        "SetupIntentConfirmParamsMandateDataCustomerAcceptance"
    ]
    """
    This hash contains details about the customer acceptance of the Mandate.
    """


class SetupIntentConfirmParamsMandateDataCustomerAcceptance(TypedDict):
    accepted_at: NotRequired[int]
    """
    The time at which the customer accepted the Mandate.
    """
    offline: NotRequired[
        "SetupIntentConfirmParamsMandateDataCustomerAcceptanceOffline"
    ]
    """
    If this is a Mandate accepted offline, this hash contains details about the offline acceptance.
    """
    online: NotRequired[
        "SetupIntentConfirmParamsMandateDataCustomerAcceptanceOnline"
    ]
    """
    If this is a Mandate accepted online, this hash contains details about the online acceptance.
    """
    type: Literal["offline", "online"]
    """
    The type of customer acceptance information included with the Mandate. One of `online` or `offline`.
    """


class SetupIntentConfirmParamsMandateDataCustomerAcceptanceOffline(TypedDict):
    pass


class SetupIntentConfirmParamsMandateDataCustomerAcceptanceOnline(TypedDict):
    ip_address: NotRequired[str]
    """
    The IP address from which the Mandate was accepted by the customer.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the browser from which the Mandate was accepted by the customer.
    """


class SetupIntentConfirmParamsPaymentMethodData(TypedDict):
    acss_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataAcssDebit"
    ]
    """
    If this is an `acss_debit` PaymentMethod, this hash contains details about the ACSS Debit payment method.
    """
    affirm: NotRequired["SetupIntentConfirmParamsPaymentMethodDataAffirm"]
    """
    If this is an `affirm` PaymentMethod, this hash contains details about the Affirm payment method.
    """
    afterpay_clearpay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataAfterpayClearpay"
    ]
    """
    If this is an `AfterpayClearpay` PaymentMethod, this hash contains details about the AfterpayClearpay payment method.
    """
    alipay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataAlipay"]
    """
    If this is an `Alipay` PaymentMethod, this hash contains details about the Alipay payment method.
    """
    allow_redisplay: NotRequired[Literal["always", "limited", "unspecified"]]
    """
    This field indicates whether this payment method can be shown again to its customer in a checkout flow. Stripe products such as Checkout and Elements use this field to determine whether a payment method can be shown as a saved payment method in a checkout flow. The field defaults to `unspecified`.
    """
    alma: NotRequired["SetupIntentConfirmParamsPaymentMethodDataAlma"]
    """
    If this is a Alma PaymentMethod, this hash contains details about the Alma payment method.
    """
    amazon_pay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataAmazonPay"
    ]
    """
    If this is a AmazonPay PaymentMethod, this hash contains details about the AmazonPay payment method.
    """
    au_becs_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataAuBecsDebit"
    ]
    """
    If this is an `au_becs_debit` PaymentMethod, this hash contains details about the bank account.
    """
    bacs_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataBacsDebit"
    ]
    """
    If this is a `bacs_debit` PaymentMethod, this hash contains details about the Bacs Direct Debit bank account.
    """
    bancontact: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataBancontact"
    ]
    """
    If this is a `bancontact` PaymentMethod, this hash contains details about the Bancontact payment method.
    """
    billie: NotRequired["SetupIntentConfirmParamsPaymentMethodDataBillie"]
    """
    If this is a `billie` PaymentMethod, this hash contains details about the Billie payment method.
    """
    billing_details: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataBillingDetails"
    ]
    """
    Billing information associated with the PaymentMethod that may be used or required by particular types of payment methods.
    """
    bizum: NotRequired["SetupIntentConfirmParamsPaymentMethodDataBizum"]
    """
    If this is a `bizum` PaymentMethod, this hash contains details about the Bizum payment method.
    """
    blik: NotRequired["SetupIntentConfirmParamsPaymentMethodDataBlik"]
    """
    If this is a `blik` PaymentMethod, this hash contains details about the BLIK payment method.
    """
    boleto: NotRequired["SetupIntentConfirmParamsPaymentMethodDataBoleto"]
    """
    If this is a `boleto` PaymentMethod, this hash contains details about the Boleto payment method.
    """
    cashapp: NotRequired["SetupIntentConfirmParamsPaymentMethodDataCashapp"]
    """
    If this is a `cashapp` PaymentMethod, this hash contains details about the Cash App Pay payment method.
    """
    crypto: NotRequired["SetupIntentConfirmParamsPaymentMethodDataCrypto"]
    """
    If this is a Crypto PaymentMethod, this hash contains details about the Crypto payment method.
    """
    customer_balance: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataCustomerBalance"
    ]
    """
    If this is a `customer_balance` PaymentMethod, this hash contains details about the CustomerBalance payment method.
    """
    eps: NotRequired["SetupIntentConfirmParamsPaymentMethodDataEps"]
    """
    If this is an `eps` PaymentMethod, this hash contains details about the EPS payment method.
    """
    fpx: NotRequired["SetupIntentConfirmParamsPaymentMethodDataFpx"]
    """
    If this is an `fpx` PaymentMethod, this hash contains details about the FPX payment method.
    """
    gift_card: NotRequired["SetupIntentConfirmParamsPaymentMethodDataGiftCard"]
    """
    If this is a `gift_card` PaymentMethod, this hash contains details about the gift card payment method.
    """
    giropay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataGiropay"]
    """
    If this is a `giropay` PaymentMethod, this hash contains details about the Giropay payment method.
    """
    gopay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataGopay"]
    """
    If this is a Gopay PaymentMethod, this hash contains details about the Gopay payment method.
    """
    grabpay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataGrabpay"]
    """
    If this is a `grabpay` PaymentMethod, this hash contains details about the GrabPay payment method.
    """
    id_bank_transfer: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataIdBankTransfer"
    ]
    """
    If this is an `IdBankTransfer` PaymentMethod, this hash contains details about the IdBankTransfer payment method.
    """
    ideal: NotRequired["SetupIntentConfirmParamsPaymentMethodDataIdeal"]
    """
    If this is an `ideal` PaymentMethod, this hash contains details about the iDEAL payment method.
    """
    interac_present: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataInteracPresent"
    ]
    """
    If this is an `interac_present` PaymentMethod, this hash contains details about the Interac Present payment method.
    """
    kakao_pay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataKakaoPay"]
    """
    If this is a `kakao_pay` PaymentMethod, this hash contains details about the Kakao Pay payment method.
    """
    klarna: NotRequired["SetupIntentConfirmParamsPaymentMethodDataKlarna"]
    """
    If this is a `klarna` PaymentMethod, this hash contains details about the Klarna payment method.
    """
    konbini: NotRequired["SetupIntentConfirmParamsPaymentMethodDataKonbini"]
    """
    If this is a `konbini` PaymentMethod, this hash contains details about the Konbini payment method.
    """
    kr_card: NotRequired["SetupIntentConfirmParamsPaymentMethodDataKrCard"]
    """
    If this is a `kr_card` PaymentMethod, this hash contains details about the Korean Card payment method.
    """
    link: NotRequired["SetupIntentConfirmParamsPaymentMethodDataLink"]
    """
    If this is an `Link` PaymentMethod, this hash contains details about the Link payment method (Link is also known as Onelink in the UK).
    """
    mb_way: NotRequired["SetupIntentConfirmParamsPaymentMethodDataMbWay"]
    """
    If this is a MB WAY PaymentMethod, this hash contains details about the MB WAY payment method.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    mobilepay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataMobilepay"
    ]
    """
    If this is a `mobilepay` PaymentMethod, this hash contains details about the MobilePay payment method.
    """
    multibanco: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataMultibanco"
    ]
    """
    If this is a `multibanco` PaymentMethod, this hash contains details about the Multibanco payment method.
    """
    naver_pay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataNaverPay"]
    """
    If this is a `naver_pay` PaymentMethod, this hash contains details about the Naver Pay payment method.
    """
    nz_bank_account: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataNzBankAccount"
    ]
    """
    If this is an nz_bank_account PaymentMethod, this hash contains details about the nz_bank_account payment method.
    """
    oxxo: NotRequired["SetupIntentConfirmParamsPaymentMethodDataOxxo"]
    """
    If this is an `oxxo` PaymentMethod, this hash contains details about the OXXO payment method.
    """
    p24: NotRequired["SetupIntentConfirmParamsPaymentMethodDataP24"]
    """
    If this is a `p24` PaymentMethod, this hash contains details about the P24 payment method.
    """
    pay_by_bank: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataPayByBank"
    ]
    """
    If this is a `pay_by_bank` PaymentMethod, this hash contains details about the PayByBank payment method.
    """
    payco: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPayco"]
    """
    If this is a `payco` PaymentMethod, this hash contains details about the PAYCO payment method.
    """
    paynow: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPaynow"]
    """
    If this is a `paynow` PaymentMethod, this hash contains details about the PayNow payment method.
    """
    paypal: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPaypal"]
    """
    If this is a `paypal` PaymentMethod, this hash contains details about the PayPal payment method.
    """
    paypay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPaypay"]
    """
    If this is a `paypay` PaymentMethod, this hash contains details about the PayPay payment method.
    """
    payto: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPayto"]
    """
    If this is a `payto` PaymentMethod, this hash contains details about the PayTo payment method.
    """
    pix: NotRequired["SetupIntentConfirmParamsPaymentMethodDataPix"]
    """
    If this is a `pix` PaymentMethod, this hash contains details about the Pix payment method.
    """
    promptpay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataPromptpay"
    ]
    """
    If this is a `promptpay` PaymentMethod, this hash contains details about the PromptPay payment method.
    """
    qris: NotRequired["SetupIntentConfirmParamsPaymentMethodDataQris"]
    """
    If this is a `qris` PaymentMethod, this hash contains details about the QRIS payment method.
    """
    radar_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataRadarOptions"
    ]
    """
    Options to configure Radar. See [Radar Session](https://docs.stripe.com/radar/radar-session) for more information.
    """
    rechnung: NotRequired["SetupIntentConfirmParamsPaymentMethodDataRechnung"]
    """
    If this is a `rechnung` PaymentMethod, this hash contains details about the Rechnung payment method.
    """
    revolut_pay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataRevolutPay"
    ]
    """
    If this is a `revolut_pay` PaymentMethod, this hash contains details about the Revolut Pay payment method.
    """
    samsung_pay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataSamsungPay"
    ]
    """
    If this is a `samsung_pay` PaymentMethod, this hash contains details about the SamsungPay payment method.
    """
    satispay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataSatispay"]
    """
    If this is a `satispay` PaymentMethod, this hash contains details about the Satispay payment method.
    """
    scalapay: NotRequired["SetupIntentConfirmParamsPaymentMethodDataScalapay"]
    """
    If this is a Scalapay PaymentMethod, this hash contains details about the Scalapay payment method.
    """
    sepa_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataSepaDebit"
    ]
    """
    If this is a `sepa_debit` PaymentMethod, this hash contains details about the SEPA debit bank account.
    """
    shared_payment_granted_token: NotRequired[str]
    """
    ID of the SharedPaymentGrantedToken used to confirm this PaymentIntent.
    """
    shopeepay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataShopeepay"
    ]
    """
    If this is a Shopeepay PaymentMethod, this hash contains details about the Shopeepay payment method.
    """
    sofort: NotRequired["SetupIntentConfirmParamsPaymentMethodDataSofort"]
    """
    If this is a `sofort` PaymentMethod, this hash contains details about the SOFORT payment method.
    """
    stripe_balance: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataStripeBalance"
    ]
    """
    This hash contains details about the Stripe balance payment method.
    """
    sunbit: NotRequired["SetupIntentConfirmParamsPaymentMethodDataSunbit"]
    """
    If this is a `sunbit` PaymentMethod, this hash contains details about the Sunbit payment method.
    """
    swish: NotRequired["SetupIntentConfirmParamsPaymentMethodDataSwish"]
    """
    If this is a `swish` PaymentMethod, this hash contains details about the Swish payment method.
    """
    tamara: NotRequired["SetupIntentConfirmParamsPaymentMethodDataTamara"]
    """
    If this is a `tamara` PaymentMethod, this hash contains details about the Tamara payment method.
    """
    twint: NotRequired["SetupIntentConfirmParamsPaymentMethodDataTwint"]
    """
    If this is a TWINT PaymentMethod, this hash contains details about the TWINT payment method.
    """
    type: Literal[
        "acss_debit",
        "affirm",
        "afterpay_clearpay",
        "alipay",
        "alma",
        "amazon_pay",
        "au_becs_debit",
        "bacs_debit",
        "bancontact",
        "billie",
        "bizum",
        "blik",
        "boleto",
        "cashapp",
        "crypto",
        "customer_balance",
        "eps",
        "fpx",
        "gift_card",
        "giropay",
        "gopay",
        "grabpay",
        "id_bank_transfer",
        "ideal",
        "kakao_pay",
        "klarna",
        "konbini",
        "kr_card",
        "link",
        "mb_way",
        "mobilepay",
        "multibanco",
        "naver_pay",
        "nz_bank_account",
        "oxxo",
        "p24",
        "pay_by_bank",
        "payco",
        "paynow",
        "paypal",
        "paypay",
        "payto",
        "pix",
        "promptpay",
        "qris",
        "rechnung",
        "revolut_pay",
        "samsung_pay",
        "satispay",
        "scalapay",
        "sepa_debit",
        "shopeepay",
        "sofort",
        "stripe_balance",
        "sunbit",
        "swish",
        "tamara",
        "twint",
        "upi",
        "us_bank_account",
        "wechat_pay",
        "zip",
    ]
    """
    The type of the PaymentMethod. An additional hash is included on the PaymentMethod with a name matching this value. It contains additional information specific to the PaymentMethod type.
    """
    upi: NotRequired["SetupIntentConfirmParamsPaymentMethodDataUpi"]
    """
    If this is a `upi` PaymentMethod, this hash contains details about the UPI payment method.
    """
    us_bank_account: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataUsBankAccount"
    ]
    """
    If this is an `us_bank_account` PaymentMethod, this hash contains details about the US bank account payment method.
    """
    wechat_pay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataWechatPay"
    ]
    """
    If this is an `wechat_pay` PaymentMethod, this hash contains details about the wechat_pay payment method.
    """
    zip: NotRequired["SetupIntentConfirmParamsPaymentMethodDataZip"]
    """
    If this is a `zip` PaymentMethod, this hash contains details about the Zip payment method.
    """


class SetupIntentConfirmParamsPaymentMethodDataAcssDebit(TypedDict):
    account_number: str
    """
    Customer's bank account number.
    """
    institution_number: str
    """
    Institution number of the customer's bank.
    """
    transit_number: str
    """
    Transit number of the customer's bank.
    """


class SetupIntentConfirmParamsPaymentMethodDataAffirm(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataAfterpayClearpay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataAlipay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataAlma(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataAmazonPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataAuBecsDebit(TypedDict):
    account_number: str
    """
    The account number for the bank account.
    """
    bsb_number: str
    """
    Bank-State-Branch number of the bank account.
    """


class SetupIntentConfirmParamsPaymentMethodDataBacsDebit(TypedDict):
    account_number: NotRequired[str]
    """
    Account number of the bank account that the funds will be debited from.
    """
    sort_code: NotRequired[str]
    """
    Sort code of the bank account. (e.g., `10-20-30`)
    """


class SetupIntentConfirmParamsPaymentMethodDataBancontact(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataBillie(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataBillingDetails(TypedDict):
    address: NotRequired[
        "Literal['']|SetupIntentConfirmParamsPaymentMethodDataBillingDetailsAddress"
    ]
    """
    Billing address.
    """
    email: NotRequired["Literal['']|str"]
    """
    Email address.
    """
    name: NotRequired["Literal['']|str"]
    """
    Full name.
    """
    phone: NotRequired["Literal['']|str"]
    """
    Billing phone number (including extension).
    """
    tax_id: NotRequired[str]
    """
    Taxpayer identification number. Used only for transactions between LATAM buyers and non-LATAM sellers.
    """


class SetupIntentConfirmParamsPaymentMethodDataBillingDetailsAddress(
    TypedDict
):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class SetupIntentConfirmParamsPaymentMethodDataBizum(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataBlik(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataBoleto(TypedDict):
    tax_id: str
    """
    The tax ID of the customer (CPF for individual consumers or CNPJ for businesses consumers)
    """


class SetupIntentConfirmParamsPaymentMethodDataCashapp(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataCrypto(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataCustomerBalance(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataEps(TypedDict):
    bank: NotRequired[
        Literal[
            "arzte_und_apotheker_bank",
            "austrian_anadi_bank_ag",
            "bank_austria",
            "bankhaus_carl_spangler",
            "bankhaus_schelhammer_und_schattera_ag",
            "bawag_psk_ag",
            "bks_bank_ag",
            "brull_kallmus_bank_ag",
            "btv_vier_lander_bank",
            "capital_bank_grawe_gruppe_ag",
            "deutsche_bank_ag",
            "dolomitenbank",
            "easybank_ag",
            "erste_bank_und_sparkassen",
            "hypo_alpeadriabank_international_ag",
            "hypo_bank_burgenland_aktiengesellschaft",
            "hypo_noe_lb_fur_niederosterreich_u_wien",
            "hypo_oberosterreich_salzburg_steiermark",
            "hypo_tirol_bank_ag",
            "hypo_vorarlberg_bank_ag",
            "marchfelder_bank",
            "oberbank_ag",
            "raiffeisen_bankengruppe_osterreich",
            "schoellerbank_ag",
            "sparda_bank_wien",
            "volksbank_gruppe",
            "volkskreditbank_ag",
            "vr_bank_braunau",
        ]
    ]
    """
    The customer's bank.
    """


class SetupIntentConfirmParamsPaymentMethodDataFpx(TypedDict):
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    Account holder type for FPX transaction
    """
    bank: Literal[
        "affin_bank",
        "agrobank",
        "alliance_bank",
        "ambank",
        "bank_islam",
        "bank_muamalat",
        "bank_of_china",
        "bank_rakyat",
        "bsn",
        "cimb",
        "deutsche_bank",
        "hong_leong_bank",
        "hsbc",
        "kfh",
        "maybank2e",
        "maybank2u",
        "ocbc",
        "pb_enterprise",
        "public_bank",
        "rhb",
        "standard_chartered",
        "uob",
    ]
    """
    The customer's bank.
    """


class SetupIntentConfirmParamsPaymentMethodDataGiftCard(TypedDict):
    gift_card: str
    """
    The gift card ID to redeem
    """


class SetupIntentConfirmParamsPaymentMethodDataGiropay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataGopay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataGrabpay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataIdBankTransfer(TypedDict):
    bank: NotRequired[Literal["bca", "bni", "bri", "cimb", "permata"]]
    """
    Bank where the account is held.
    """


class SetupIntentConfirmParamsPaymentMethodDataIdeal(TypedDict):
    bank: NotRequired[
        Literal[
            "abn_amro",
            "adyen",
            "asn_bank",
            "bunq",
            "buut",
            "finom",
            "handelsbanken",
            "ing",
            "knab",
            "mollie",
            "moneyou",
            "n26",
            "nn",
            "rabobank",
            "regiobank",
            "revolut",
            "sns_bank",
            "triodos_bank",
            "van_lanschot",
            "yoursafe",
        ]
    ]
    """
    The customer's bank. Only use this parameter for existing customers. Don't use it for new customers.
    """


class SetupIntentConfirmParamsPaymentMethodDataInteracPresent(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataKakaoPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataKlarna(TypedDict):
    dob: NotRequired["SetupIntentConfirmParamsPaymentMethodDataKlarnaDob"]
    """
    Customer's date of birth
    """


class SetupIntentConfirmParamsPaymentMethodDataKlarnaDob(TypedDict):
    day: int
    """
    The day of birth, between 1 and 31.
    """
    month: int
    """
    The month of birth, between 1 and 12.
    """
    year: int
    """
    The four-digit year of birth.
    """


class SetupIntentConfirmParamsPaymentMethodDataKonbini(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataKrCard(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataLink(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataMbWay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataMobilepay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataMultibanco(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataNaverPay(TypedDict):
    funding: NotRequired[Literal["card", "points"]]
    """
    Whether to use Naver Pay points or a card to fund this transaction. If not provided, this defaults to `card`.
    """


class SetupIntentConfirmParamsPaymentMethodDataNzBankAccount(TypedDict):
    account_holder_name: NotRequired[str]
    """
    The name on the bank account. Only required if the account holder name is different from the name of the authorized signatory collected in the PaymentMethod's billing details.
    """
    account_number: str
    """
    The account number for the bank account.
    """
    bank_code: str
    """
    The numeric code for the bank account's bank.
    """
    branch_code: str
    """
    The numeric code for the bank account's bank branch.
    """
    reference: NotRequired[str]
    suffix: str
    """
    The suffix of the bank account number.
    """


class SetupIntentConfirmParamsPaymentMethodDataOxxo(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataP24(TypedDict):
    bank: NotRequired[
        Literal[
            "alior_bank",
            "bank_millennium",
            "bank_nowy_bfg_sa",
            "bank_pekao_sa",
            "banki_spbdzielcze",
            "blik",
            "bnp_paribas",
            "boz",
            "citi_handlowy",
            "credit_agricole",
            "envelobank",
            "etransfer_pocztowy24",
            "getin_bank",
            "ideabank",
            "ing",
            "inteligo",
            "mbank_mtransfer",
            "nest_przelew",
            "noble_pay",
            "pbac_z_ipko",
            "plus_bank",
            "santander_przelew24",
            "tmobile_usbugi_bankowe",
            "toyota_bank",
            "velobank",
            "volkswagen_bank",
        ]
    ]
    """
    The customer's bank.
    """


class SetupIntentConfirmParamsPaymentMethodDataPayByBank(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPayco(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPaynow(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPaypal(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPaypay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPayto(TypedDict):
    account_number: NotRequired[str]
    """
    The account number for the bank account.
    """
    bsb_number: NotRequired[str]
    """
    Bank-State-Branch number of the bank account.
    """
    pay_id: NotRequired[str]
    """
    The PayID alias for the bank account.
    """


class SetupIntentConfirmParamsPaymentMethodDataPix(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataPromptpay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataQris(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataRadarOptions(TypedDict):
    session: NotRequired[str]
    """
    A [Radar Session](https://docs.stripe.com/radar/radar-session) is a snapshot of the browser metadata and device details that help Radar make more accurate predictions on your payments.
    """


class SetupIntentConfirmParamsPaymentMethodDataRechnung(TypedDict):
    dob: "SetupIntentConfirmParamsPaymentMethodDataRechnungDob"
    """
    Customer's date of birth
    """


class SetupIntentConfirmParamsPaymentMethodDataRechnungDob(TypedDict):
    day: int
    """
    The day of birth, between 1 and 31.
    """
    month: int
    """
    The month of birth, between 1 and 12.
    """
    year: int
    """
    The four-digit year of birth.
    """


class SetupIntentConfirmParamsPaymentMethodDataRevolutPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataSamsungPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataSatispay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataScalapay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataSepaDebit(TypedDict):
    iban: str
    """
    IBAN of the bank account.
    """


class SetupIntentConfirmParamsPaymentMethodDataShopeepay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataSofort(TypedDict):
    country: Literal["AT", "BE", "DE", "ES", "IT", "NL"]
    """
    Two-letter ISO code representing the country the bank account is located in.
    """


class SetupIntentConfirmParamsPaymentMethodDataStripeBalance(TypedDict):
    account: NotRequired[str]
    """
    The connected account ID whose Stripe balance to use as the source of payment
    """


class SetupIntentConfirmParamsPaymentMethodDataSunbit(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataSwish(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataTamara(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataTwint(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataUpi(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodDataUpiMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate
    """


class SetupIntentConfirmParamsPaymentMethodDataUpiMandateOptions(TypedDict):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """
    end_date: NotRequired[int]
    """
    End date of the mandate or subscription.
    """


class SetupIntentConfirmParamsPaymentMethodDataUsBankAccount(TypedDict):
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    Account holder type: individual or company.
    """
    account_number: NotRequired[str]
    """
    Account number of the bank account.
    """
    account_type: NotRequired[Literal["checking", "savings"]]
    """
    Account type: checkings or savings. Defaults to checking if omitted.
    """
    financial_connections_account: NotRequired[str]
    """
    The ID of a Financial Connections Account to use as a payment method.
    """
    routing_number: NotRequired[str]
    """
    Routing number of the bank account.
    """


class SetupIntentConfirmParamsPaymentMethodDataWechatPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodDataZip(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodOptions(TypedDict):
    acss_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsAcssDebit"
    ]
    """
    If this is a `acss_debit` SetupIntent, this sub-hash contains details about the ACSS Debit payment method options.
    """
    amazon_pay: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsAmazonPay"
    ]
    """
    If this is a `amazon_pay` SetupIntent, this sub-hash contains details about the AmazonPay payment method options.
    """
    bacs_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsBacsDebit"
    ]
    """
    If this is a `bacs_debit` SetupIntent, this sub-hash contains details about the Bacs Debit payment method options.
    """
    bizum: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsBizum"]
    """
    If this is a `bizum` SetupIntent, this sub-hash contains details about the Bizum payment method options.
    """
    card: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsCard"]
    """
    Configuration for any card setup attempted on this SetupIntent.
    """
    card_present: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsCardPresent"
    ]
    """
    If this is a `card_present` PaymentMethod, this sub-hash contains details about the card-present payment method options.
    """
    klarna: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsKlarna"]
    """
    If this is a `klarna` PaymentMethod, this hash contains details about the Klarna payment method options.
    """
    link: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsLink"]
    """
    If this is a `link` PaymentMethod, this sub-hash contains details about the Link payment method options (Link is also known as Onelink in the UK).
    """
    paypal: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsPaypal"]
    """
    If this is a `paypal` PaymentMethod, this sub-hash contains details about the PayPal payment method options.
    """
    payto: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsPayto"]
    """
    If this is a `payto` SetupIntent, this sub-hash contains details about the PayTo payment method options.
    """
    pix: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsPix"]
    """
    If this is a `pix` SetupIntent, this sub-hash contains details about the Pix payment method options.
    """
    sepa_debit: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsSepaDebit"
    ]
    """
    If this is a `sepa_debit` SetupIntent, this sub-hash contains details about the SEPA Debit payment method options.
    """
    stripe_balance: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsStripeBalance"
    ]
    """
    If this is a `stripe_balance` PaymentMethod, this sub-hash contains details about the Stripe Balance payment method options.
    """
    upi: NotRequired["SetupIntentConfirmParamsPaymentMethodOptionsUpi"]
    """
    If this is a `upi` SetupIntent, this sub-hash contains details about the UPI payment method options.
    """
    us_bank_account: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccount"
    ]
    """
    If this is a `us_bank_account` SetupIntent, this sub-hash contains details about the US bank account payment method options.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsAcssDebit(TypedDict):
    currency: NotRequired[Literal["cad", "usd"]]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsAcssDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Bank account verification method. The default value is `automatic`.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsAcssDebitMandateOptions(
    TypedDict,
):
    custom_mandate_url: NotRequired["Literal['']|str"]
    """
    A URL for custom mandate text to render during confirmation step.
    The URL will be rendered with additional GET parameters `payment_intent` and `payment_intent_client_secret` when confirming a Payment Intent,
    or `setup_intent` and `setup_intent_client_secret` when confirming a Setup Intent.
    """
    default_for: NotRequired[List[Literal["invoice", "subscription"]]]
    """
    List of Stripe products where this mandate can be selected automatically.
    """
    interval_description: NotRequired[str]
    """
    Description of the mandate interval. Only required if 'payment_schedule' parameter is 'interval' or 'combined'.
    """
    payment_schedule: NotRequired[Literal["combined", "interval", "sporadic"]]
    """
    Payment schedule for the mandate.
    """
    transaction_type: NotRequired[Literal["business", "personal"]]
    """
    Transaction type of the mandate.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsAmazonPay(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodOptionsBacsDebit(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsBacsDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """


class SetupIntentConfirmParamsPaymentMethodOptionsBacsDebitMandateOptions(
    TypedDict,
):
    reference_prefix: NotRequired["Literal['']|str"]
    """
    Prefix used to generate the Mandate reference. Must be at most 12 characters long. Must consist of only uppercase letters, numbers, spaces, or the following special characters: '/', '_', '-', '&', '.'. Cannot begin with 'DDIC' or 'STRIPE'.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsBizum(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodOptionsCard(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsCardMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate for cards issued in India.
    """
    moto: NotRequired[bool]
    """
    When specified, this parameter signals that a card has been collected
    as MOTO (Mail Order Telephone Order) and thus out of scope for SCA. This
    parameter can only be provided during confirmation.
    """
    network: NotRequired[
        Literal[
            "amex",
            "cartes_bancaires",
            "diners",
            "discover",
            "eftpos_au",
            "girocard",
            "interac",
            "jcb",
            "link",
            "mastercard",
            "unionpay",
            "unknown",
            "visa",
        ]
    ]
    """
    Selected network to process this SetupIntent on. Depends on the available networks of the card attached to the SetupIntent. Can be only set confirm-time.
    """
    request_three_d_secure: NotRequired[
        Literal["any", "automatic", "challenge"]
    ]
    """
    We strongly recommend that you rely on our SCA Engine to automatically prompt your customers for authentication based on risk level and [other requirements](https://docs.stripe.com/strong-customer-authentication). However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option. If not provided, this value defaults to `automatic`. Read our guide on [manually requesting 3D Secure](https://docs.stripe.com/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
    """
    three_d_secure: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecure"
    ]
    """
    If 3D Secure authentication was performed with a third-party provider,
    the authentication details to use for this setup.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsCardMandateOptions(
    TypedDict
):
    amount: int
    """
    Amount to be charged for future payments, specified in the presentment currency.
    """
    amount_type: Literal["fixed", "maximum"]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    currency: str
    """
    Currency in which future payments will be charged. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """
    end_date: NotRequired[int]
    """
    End date of the mandate or subscription. If not provided, the mandate will be active until canceled. If provided, end date should be after start date.
    """
    interval: Literal["day", "month", "sporadic", "week", "year"]
    """
    Specifies payment frequency. One of `day`, `week`, `month`, `year`, or `sporadic`.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals between payments. For example, `interval=month` and `interval_count=3` indicates one payment every three months. Maximum of one year interval allowed (1 year, 12 months, or 52 weeks). This parameter is optional when `interval=sporadic`.
    """
    reference: str
    """
    Unique identifier for the mandate or subscription.
    """
    start_date: int
    """
    Start date of the mandate or subscription. Start date should not be lesser than yesterday.
    """
    supported_types: NotRequired[List[Literal["india"]]]
    """
    Specifies the type of mandates supported. Possible values are `india`.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecure(TypedDict):
    ares_trans_status: NotRequired[Literal["A", "C", "I", "N", "R", "U", "Y"]]
    """
    The `transStatus` returned from the card Issuer's ACS in the ARes.
    """
    cryptogram: NotRequired[str]
    """
    The cryptogram, also known as the "authentication value" (AAV, CAVV or
    AEVV). This value is 20 bytes, base64-encoded into a 28-character string.
    (Most 3D Secure providers will return the base64-encoded version, which
    is what you should specify here.)
    """
    electronic_commerce_indicator: NotRequired[
        Literal["01", "02", "05", "06", "07"]
    ]
    """
    The Electronic Commerce Indicator (ECI) is returned by your 3D Secure
    provider and indicates what degree of authentication was performed.
    """
    network_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecureNetworkOptions"
    ]
    """
    Network specific 3DS fields. Network specific arguments require an
    explicit card brand choice. The parameter `payment_method_options.card.network``
    must be populated accordingly
    """
    requestor_challenge_indicator: NotRequired[str]
    """
    The challenge indicator (`threeDSRequestorChallengeInd`) which was requested in the
    AReq sent to the card Issuer's ACS. A string containing 2 digits from 01-99.
    """
    transaction_id: NotRequired[str]
    """
    For 3D Secure 1, the XID. For 3D Secure 2, the Directory Server
    Transaction ID (dsTransID).
    """
    version: NotRequired[Literal["1.0.2", "2.1.0", "2.2.0", "2.3.0", "2.3.1"]]
    """
    The version of 3D Secure that was performed.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecureNetworkOptions(
    TypedDict,
):
    cartes_bancaires: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecureNetworkOptionsCartesBancaires"
    ]
    """
    Cartes Bancaires-specific 3DS fields.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsCardThreeDSecureNetworkOptionsCartesBancaires(
    TypedDict,
):
    cb_avalgo: Literal["0", "1", "2", "3", "4", "A"]
    """
    The cryptogram calculation algorithm used by the card Issuer's ACS
    to calculate the Authentication cryptogram. Also known as `cavvAlgorithm`.
    messageExtension: CB-AVALGO
    """
    cb_exemption: NotRequired[str]
    """
    The exemption indicator returned from Cartes Bancaires in the ARes.
    message extension: CB-EXEMPTION; string (4 characters)
    This is a 3 byte bitmap (low significant byte first and most significant
    bit first) that has been Base64 encoded
    """
    cb_score: NotRequired[int]
    """
    The risk score returned from Cartes Bancaires in the ARes.
    message extension: CB-SCORE; numeric value 0-99
    """


class SetupIntentConfirmParamsPaymentMethodOptionsCardPresent(TypedDict):
    pass


class SetupIntentConfirmParamsPaymentMethodOptionsKlarna(TypedDict):
    currency: NotRequired[str]
    """
    The currency of the SetupIntent. Three letter ISO currency code.
    """
    on_demand: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsKlarnaOnDemand"
    ]
    """
    On-demand details if setting up a payment method for on-demand payments.
    """
    preferred_locale: NotRequired[
        Literal[
            "cs-CZ",
            "da-DK",
            "de-AT",
            "de-CH",
            "de-DE",
            "el-GR",
            "en-AT",
            "en-AU",
            "en-BE",
            "en-CA",
            "en-CH",
            "en-CZ",
            "en-DE",
            "en-DK",
            "en-ES",
            "en-FI",
            "en-FR",
            "en-GB",
            "en-GR",
            "en-IE",
            "en-IT",
            "en-NL",
            "en-NO",
            "en-NZ",
            "en-PL",
            "en-PT",
            "en-RO",
            "en-SE",
            "en-US",
            "es-ES",
            "es-US",
            "fi-FI",
            "fr-BE",
            "fr-CA",
            "fr-CH",
            "fr-FR",
            "it-CH",
            "it-IT",
            "nb-NO",
            "nl-BE",
            "nl-NL",
            "pl-PL",
            "pt-PT",
            "ro-RO",
            "sv-FI",
            "sv-SE",
        ]
    ]
    """
    Preferred language of the Klarna authorization page that the customer is redirected to
    """
    subscriptions: NotRequired[
        "Literal['']|List[SetupIntentConfirmParamsPaymentMethodOptionsKlarnaSubscription]"
    ]
    """
    Subscription details if setting up or charging a subscription
    """


class SetupIntentConfirmParamsPaymentMethodOptionsKlarnaOnDemand(TypedDict):
    average_amount: NotRequired[int]
    """
    Your average amount value. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    maximum_amount: NotRequired[int]
    """
    The maximum value you may charge a customer per purchase. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    minimum_amount: NotRequired[int]
    """
    The lowest or minimum value you may charge a customer per purchase. You can use a value across your customer base, or segment based on customer type, country, etc.
    """
    purchase_interval: NotRequired[Literal["day", "month", "week", "year"]]
    """
    Interval at which the customer is making purchases
    """
    purchase_interval_count: NotRequired[int]
    """
    The number of `purchase_interval` between charges
    """


class SetupIntentConfirmParamsPaymentMethodOptionsKlarnaSubscription(
    TypedDict
):
    interval: Literal["day", "month", "week", "year"]
    """
    Unit of time between subscription charges.
    """
    interval_count: NotRequired[int]
    """
    The number of intervals (specified in the `interval` attribute) between subscription charges. For example, `interval=month` and `interval_count=3` charges every 3 months.
    """
    name: NotRequired[str]
    """
    Name for subscription.
    """
    next_billing: "SetupIntentConfirmParamsPaymentMethodOptionsKlarnaSubscriptionNextBilling"
    """
    Describes the upcoming charge for this subscription.
    """
    reference: str
    """
    A non-customer-facing reference to correlate subscription charges in the Klarna app. Use a value that persists across subscription charges.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsKlarnaSubscriptionNextBilling(
    TypedDict,
):
    amount: int
    """
    The amount of the next charge for the subscription.
    """
    date: str
    """
    The date of the next charge for the subscription in YYYY-MM-DD format.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsLink(TypedDict):
    persistent_token: NotRequired[str]
    """
    [Deprecated] This is a legacy parameter that no longer has any function.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsPaypal(TypedDict):
    billing_agreement_id: NotRequired[str]
    """
    The PayPal Billing Agreement ID (BAID). This is an ID generated by PayPal which represents the mandate between the merchant and the customer.
    """
    currency: NotRequired[str]
    subsellers: NotRequired[List[str]]
    """
    The Stripe connected account IDs of the sellers on the platform for this transaction (optional). Only allowed when [separate charges and transfers](https://stripe.com/docs/connect/separate-charges-and-transfers) are used.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsPayto(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsPaytoMandateOptions"
    ]
    """
    Additional fields for Mandate creation.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsPaytoMandateOptions(
    TypedDict,
):
    amount: NotRequired["Literal['']|int"]
    """
    Amount that will be collected. It is required when `amount_type` is `fixed`.
    """
    amount_type: NotRequired["Literal['']|Literal['fixed', 'maximum']"]
    """
    The type of amount that will be collected. The amount charged must be exact or up to the value of `amount` param for `fixed` or `maximum` type respectively. Defaults to `maximum`.
    """
    end_date: NotRequired["Literal['']|str"]
    """
    Date, in YYYY-MM-DD format, after which payments will not be collected. Defaults to no end date.
    """
    payment_schedule: NotRequired[
        "Literal['']|Literal['adhoc', 'annual', 'daily', 'fortnightly', 'monthly', 'quarterly', 'semi_annual', 'weekly']"
    ]
    """
    The periodicity at which payments will be collected. Defaults to `adhoc`.
    """
    payments_per_period: NotRequired["Literal['']|int"]
    """
    The number of payments that will be made during a payment period. Defaults to 1 except for when `payment_schedule` is `adhoc`. In that case, it defaults to no limit.
    """
    purpose: NotRequired[
        "Literal['']|Literal['dependant_support', 'government', 'loan', 'mortgage', 'other', 'pension', 'personal', 'retail', 'salary', 'tax', 'utility']"
    ]
    """
    The purpose for which payments are made. Has a default value based on your merchant category code.
    """
    start_date: NotRequired["Literal['']|str"]
    """
    Date, in YYYY-MM-DD format, from which payments will be collected. Defaults to confirmation time.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsPix(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsPixMandateOptions"
    ]
    """
    Additional fields for mandate creation.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsPixMandateOptions(TypedDict):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments. Required when `amount_type=fixed`. If not provided for `amount_type=maximum`, defaults to 40000.
    """
    amount_includes_iof: NotRequired[Literal["always", "never"]]
    """
    Determines if the amount includes the IOF tax. Defaults to `never`.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    Type of amount. Defaults to `maximum`.
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Only `brl` is supported currently.
    """
    end_date: NotRequired[str]
    """
    Date when the mandate expires and no further payments will be charged, in `YYYY-MM-DD`. If not provided, the mandate will be active until canceled. If provided, end date should be after start date.
    """
    payment_schedule: NotRequired[
        Literal["halfyearly", "monthly", "quarterly", "weekly", "yearly"]
    ]
    """
    Schedule at which the future payments will be charged. Defaults to `monthly`.
    """
    reference: NotRequired[str]
    """
    Subscription name displayed to buyers in their bank app. Defaults to the displayable business name.
    """
    start_date: NotRequired[str]
    """
    Start date of the mandate, in `YYYY-MM-DD`. Start date should be at least 3 days in the future. Defaults to 3 days after the current date.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsSepaDebit(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsSepaDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """


class SetupIntentConfirmParamsPaymentMethodOptionsSepaDebitMandateOptions(
    TypedDict,
):
    reference_prefix: NotRequired["Literal['']|str"]
    """
    Prefix used to generate the Mandate reference. Must be at most 12 characters long. Must consist of only uppercase letters, numbers, spaces, or the following special characters: '/', '_', '-', '&', '.'. Cannot begin with 'STRIPE'.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsStripeBalance(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsStripeBalanceMandateOptions"
    ]
    """
    Additional fields for mandate creation.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsStripeBalanceMandateOptions(
    TypedDict,
):
    stripe_balance_debit_agreement: NotRequired[str]
    """
    The ID of the Stripe Balance Debit Agreement used for this mandate.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUpi(TypedDict):
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUpiMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate
    """
    setup_future_usage: NotRequired[
        "Literal['']|Literal['none', 'off_session', 'on_session']"
    ]


class SetupIntentConfirmParamsPaymentMethodOptionsUpiMandateOptions(TypedDict):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    One of `fixed` or `maximum`. If `fixed`, the `amount` param refers to the exact amount to be charged in future payments. If `maximum`, the amount charged can be up to the value passed for the `amount` param.
    """
    description: NotRequired[str]
    """
    A description of the mandate or subscription that is meant to be displayed to the customer.
    """
    end_date: NotRequired[int]
    """
    End date of the mandate or subscription.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccount(TypedDict):
    financial_connections: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnections"
    ]
    """
    Additional fields for Financial Connections Session creation
    """
    mandate_options: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    networks: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountNetworks"
    ]
    """
    Additional fields for network related functions
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Bank account verification method. The default value is `automatic`.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnections(
    TypedDict,
):
    filters: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters"
    ]
    """
    Provide filters for the linked accounts that the customer can select for the payment method.
    """
    manual_entry: NotRequired[
        "SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsManualEntry"
    ]
    """
    Customize manual entry behavior
    """
    permissions: NotRequired[
        List[
            Literal["balances", "ownership", "payment_method", "transactions"]
        ]
    ]
    """
    The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included. Valid permissions include: `balances`, `ownership`, `payment_method`, and `transactions`.
    """
    prefetch: NotRequired[
        List[
            Literal[
                "balances", "inferred_balances", "ownership", "transactions"
            ]
        ]
    ]
    """
    List of data features that you would like to retrieve upon account creation.
    """
    return_url: NotRequired[str]
    """
    For webview integrations only. Upon completing OAuth login in the native browser, the user will be redirected to this URL to return to your app.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters(
    TypedDict,
):
    account_subcategories: NotRequired[List[Literal["checking", "savings"]]]
    """
    The account subcategories to use to filter for selectable accounts. Valid subcategories are `checking` and `savings`.
    """
    institution: NotRequired[str]
    """
    ID of the institution to use to filter for selectable accounts.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsManualEntry(
    TypedDict,
):
    mode: Literal["automatic", "custom"]
    """
    Settings for configuring manual entry of account details.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountMandateOptions(
    TypedDict,
):
    collection_method: NotRequired["Literal['']|Literal['paper']"]
    """
    The method used to collect offline mandate customer acceptance.
    """


class SetupIntentConfirmParamsPaymentMethodOptionsUsBankAccountNetworks(
    TypedDict,
):
    requested: NotRequired[List[Literal["ach", "us_domestic_wire"]]]
    """
    Triggers validations to run across the selected networks
    """


class SetupIntentConfirmParamsSetupDetails(TypedDict):
    benefit: NotRequired["SetupIntentConfirmParamsSetupDetailsBenefit"]
    """
    Benefit details for this SetupIntent
    """
    location: NotRequired[str]
    """
    The ID of the Payment Location for this SetupIntent.
    """


class SetupIntentConfirmParamsSetupDetailsBenefit(TypedDict):
    fr_meal_voucher: NotRequired[
        "Literal['']|SetupIntentConfirmParamsSetupDetailsBenefitFrMealVoucher"
    ]
    """
    French meal voucher benefit details for this SetupIntent.
    """


class SetupIntentConfirmParamsSetupDetailsBenefitFrMealVoucher(TypedDict):
    enabled: NotRequired[Literal["if_payment_method_is_eligible", "never"]]
    """
    Whether to enable meal voucher benefit for this setup intent.
    """
    siret: NotRequired[str]
    """
    The 14-digit SIRET of the meal voucher acceptor.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance_settings import BalanceSettings
    from stripe._request_options import RequestOptions
    from stripe.params._balance_settings_retrieve_params import (
        BalanceSettingsRetrieveParams,
    )
    from stripe.params._balance_settings_update_params import (
        BalanceSettingsUpdateParams,
    )


class BalanceSettingsService(StripeService):
    def retrieve(
        self,
        params: Optional["BalanceSettingsRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceSettings":
        """
        Retrieves balance settings for a given connected account.
         Related guide: [Making API calls for connected accounts](https://docs.stripe.com/connect/authentication)
        """
        return cast(
            "BalanceSettings",
            self._request(
                "get",
                "/v1/balance_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        params: Optional["BalanceSettingsRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceSettings":
        """
        Retrieves balance settings for a given connected account.
         Related guide: [Making API calls for connected accounts](https://docs.stripe.com/connect/authentication)
        """
        return cast(
            "BalanceSettings",
            await self._request_async(
                "get",
                "/v1/balance_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        params: Optional["BalanceSettingsUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceSettings":
        """
        Updates balance settings for a given connected account.
         Related guide: [Making API calls for connected accounts](https://docs.stripe.com/connect/authentication)
        """
        return cast(
            "BalanceSettings",
            self._request(
                "post",
                "/v1/balance_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        params: Optional["BalanceSettingsUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BalanceSettings":
        """
        Updates balance settings for a given connected account.
         Related guide: [Making API calls for connected accounts](https://docs.stripe.com/connect/authentication)
        """
        return cast(
            "BalanceSettings",
            await self._request_async(
                "post",
                "/v1/balance_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._line_item import LineItem
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._quote_computed_upfront_line_items_list_params import (
        QuoteComputedUpfrontLineItemsListParams,
    )


class QuoteComputedUpfrontLineItemsService(StripeService):
    def list(
        self,
        quote: str,
        params: Optional["QuoteComputedUpfrontLineItemsListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[LineItem]":
        """
        When retrieving a quote, there is an includable [computed.upfront.line_items](https://stripe.com/docs/api/quotes/object#quote_object-computed-upfront-line_items) property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of upfront line items.
        """
        return cast(
            "ListObject[LineItem]",
            self._request(
                "get",
                "/v1/quotes/{quote}/computed_upfront_line_items".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        quote: str,
        params: Optional["QuoteComputedUpfrontLineItemsListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[LineItem]":
        """
        When retrieving a quote, there is an includable [computed.upfront.line_items](https://stripe.com/docs/api/quotes/object#quote_object-computed-upfront-line_items) property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of upfront line items.
        """
        return cast(
            "ListObject[LineItem]",
            await self._request_async(
                "get",
                "/v1/quotes/{quote}/computed_upfront_line_items".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core._batch_job_cancel_params import (
        BatchJobCancelParams,
    )
    from stripe.params.v2.core._batch_job_create_params import (
        BatchJobCreateParams,
    )
    from stripe.params.v2.core._batch_job_retrieve_params import (
        BatchJobRetrieveParams,
    )
    from stripe.v2.core._batch_job import BatchJob


class BatchJobService(StripeService):
    def create(
        self,
        params: "BatchJobCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Creates a new batch job.
        """
        return cast(
            "BatchJob",
            self._request(
                "post",
                "/v2/core/batch_jobs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "BatchJobCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Creates a new batch job.
        """
        return cast(
            "BatchJob",
            await self._request_async(
                "post",
                "/v2/core/batch_jobs",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["BatchJobRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Retrieves an existing batch job.
        """
        return cast(
            "BatchJob",
            self._request(
                "get",
                "/v2/core/batch_jobs/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["BatchJobRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Retrieves an existing batch job.
        """
        return cast(
            "BatchJob",
            await self._request_async(
                "get",
                "/v2/core/batch_jobs/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        id: str,
        params: Optional["BatchJobCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Cancels an existing batch job.
        """
        return cast(
            "BatchJob",
            self._request(
                "post",
                "/v2/core/batch_jobs/{id}/cancel".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: Optional["BatchJobCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "BatchJob":
        """
        Cancels an existing batch job.
        """
        return cast(
            "BatchJob",
            await self._request_async(
                "post",
                "/v2/core/batch_jobs/{id}/cancel".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired, TypedDict


class PaymentLocationCreateParams(RequestOptions):
    address: "PaymentLocationCreateParamsAddress"
    """
    The full address of the location.
    """
    business_registration: NotRequired[
        "PaymentLocationCreateParamsBusinessRegistration"
    ]
    """
    Identification numbers associated with the location.
    """
    display_name: str
    """
    A name for the location.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """


class PaymentLocationCreateParamsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class PaymentLocationCreateParamsBusinessRegistration(TypedDict):
    siret: NotRequired[str]
    """
    14-digit SIRET (Système d'identification du répertoire des établissements) number for the location.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.billing.analytics._meter_usage import MeterUsage
    from stripe.params.billing.analytics._meter_usage_retrieve_params import (
        MeterUsageRetrieveParams,
    )


class MeterUsageService(StripeService):
    def retrieve(
        self,
        params: "MeterUsageRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "MeterUsage":
        """
        Returns aggregated meter usage data for a customer within a specified time interval. The data can be grouped by various dimensions and can include multiple meters if specified.
        """
        return cast(
            "MeterUsage",
            self._request(
                "get",
                "/v1/billing/analytics/meter_usage",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        params: "MeterUsageRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "MeterUsage":
        """
        Returns aggregated meter usage data for a customer within a specified time interval. The data can be grouped by various dimensions and can include multiple meters if specified.
        """
        return cast(
            "MeterUsage",
            await self._request_async(
                "get",
                "/v1/billing/analytics/meter_usage",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class FinancialAccountStatement(StripeObject):
    """
    A Financial Account Statement represents a monthly statement for a Financial Account.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.money_management.financial_account_statement"]
    ] = "v2.money_management.financial_account_statement"

    class FilesByCurrency(StripeObject):
        class DownloadUrl(StripeObject):
            expires_at: str
            """
            The time at which the URL expires, in ISO 8601 format (UTC).
            """
            url: str
            """
            The URL to download the file.
            """

        content_type: str
        """
        The MIME type of the file.
        """
        download_url: DownloadUrl
        """
        The download URL and expiration.
        """
        size: int
        """
        The size of the file in bytes.
        """
        _inner_class_types = {"download_url": DownloadUrl}
        _field_encodings = {"size": "int64_string"}

    class Period(StripeObject):
        end_date: str
        """
        The end of the statement period (inclusive), as a UTC-aligned ISO 8601 date
        (e.g., "2026-05-31"). For example, a May 2026 statement has end_date "2026-05-31",
        meaning all transactions up to and including May 31st UTC are included.
        """
        start_date: str
        """
        The start of the statement period (inclusive), as a UTC-aligned ISO 8601 date (e.g., "2026-05-01").
        """

    created: str
    """
    Time at which the statement was created, in ISO 8601 format (UTC).
    """
    ending_balance: UntypedStripeObject[Amount]
    """
    Available balance at the end of the statement period.
    """
    files_by_currency: Optional[UntypedStripeObject[FilesByCurrency]]
    """
    Currency-specific files and file metadata. Null by default, populated by specifying include=files_by_currency in the Retrieve endpoint.
    """
    financial_account: str
    """
    The Financial Account this statement belongs to.
    """
    id: str
    """
    Unique identifier for the statement.
    """
    livemode: bool
    """
    True if the object exists in live mode, false if in test mode.
    """
    object: Literal["v2.money_management.financial_account_statement"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    period: Period
    """
    The time period covered by this statement.
    """
    restated_by: Optional[str]
    """
    The ID of the statement that replaced this one. Only present on statements that have been restated.
    """
    restated_statement: Optional[str]
    """
    The ID of the statement this one replaces. Only present on restatements.
    """
    starting_balance: UntypedStripeObject[Amount]
    """
    Available balance at the start of the statement period.
    """
    status: Literal["active", "restated"]
    """
    The status of the statement. A statement is "active" by default.
    When a statement is replaced by a restatement, its status becomes "restated".
    """
    _inner_class_types = {
        "files_by_currency": FilesByCurrency,
        "period": Period,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class PaymentEvaluationCreateParams(RequestOptions):
    client_device_metadata_details: NotRequired[
        "PaymentEvaluationCreateParamsClientDeviceMetadataDetails"
    ]
    """
    Details about the Client Device Metadata to associate with the payment evaluation.
    """
    customer_details: "PaymentEvaluationCreateParamsCustomerDetails"
    """
    Details about the customer associated with the payment evaluation.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    payment_details: "PaymentEvaluationCreateParamsPaymentDetails"
    """
    Details about the payment.
    """


class PaymentEvaluationCreateParamsClientDeviceMetadataDetails(TypedDict):
    data: NotRequired[
        "PaymentEvaluationCreateParamsClientDeviceMetadataDetailsData"
    ]
    """
    Direct client device attributes such as IP address and user agent. Use this as an alternative to radar_session when a Radar Session isn't available.
    """
    radar_session: str
    """
    ID for the Radar Session to associate with the payment evaluation. A [Radar Session](https://docs.stripe.com/radar/radar-session) is a snapshot of the browser metadata and device details that help Radar make more accurate predictions on your payments.
    """


class PaymentEvaluationCreateParamsClientDeviceMetadataDetailsData(TypedDict):
    ip: str
    """
    The IP address of the client device.
    """
    pasted_fields: NotRequired[List[Literal["cvc", "exp", "number", "zip"]]]
    """
    Pasted fields from the checkout flow.
    """
    referrer: NotRequired[str]
    """
    The referrer of the client device.
    """
    time_on_page_ms: NotRequired[int]
    """
    The time on page in milliseconds.
    """
    user_agent: str
    """
    The user agent of the client device.
    """


class PaymentEvaluationCreateParamsCustomerDetails(TypedDict):
    customer: NotRequired[str]
    """
    The ID of the customer associated with the payment evaluation.
    """
    customer_account: NotRequired[str]
    """
    The ID of the Account representing the customer associated with the payment evaluation.
    """
    email: NotRequired[str]
    """
    The customer's email address.
    """
    name: NotRequired[str]
    """
    The customer's full name or business name.
    """
    phone: NotRequired[str]
    """
    The customer's phone number.
    """


class PaymentEvaluationCreateParamsPaymentDetails(TypedDict):
    amount: int
    """
    The intended amount to collect with this payment. A positive integer representing how much to charge in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal) (for example, 100 cents to charge 1.00 USD or 100 to charge 100 Yen, a zero-decimal currency).
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    money_movement_details: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsMoneyMovementDetails"
    ]
    """
    Details about the payment's customer presence and type.
    """
    payment_method_details: (
        "PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetails"
    )
    """
    Details about the payment method to use for the payment.
    """
    shipping_details: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsShippingDetails"
    ]
    """
    Shipping details for the payment evaluation.
    """
    statement_descriptor: NotRequired[str]
    """
    Payment statement descriptor.
    """


class PaymentEvaluationCreateParamsPaymentDetailsMoneyMovementDetails(
    TypedDict,
):
    card: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsMoneyMovementDetailsCard"
    ]
    """
    Describes card money movement details for the payment evaluation.
    """
    money_movement_type: Literal["card"]
    """
    Describes the type of money movement. Currently only `card` is supported.
    """


class PaymentEvaluationCreateParamsPaymentDetailsMoneyMovementDetailsCard(
    TypedDict,
):
    customer_presence: NotRequired[Literal["off_session", "on_session"]]
    """
    Describes the presence of the customer during the payment.
    """
    payment_type: NotRequired[
        Literal["one_off", "recurring", "setup_one_off", "setup_recurring"]
    ]
    """
    Describes the type of payment.
    """


class PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetails(
    TypedDict,
):
    billing_details: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetailsBillingDetails"
    ]
    """
    Billing information associated with the payment evaluation.
    """
    payment_method: str
    """
    ID of the payment method used in this payment evaluation.
    """


class PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetailsBillingDetails(
    TypedDict,
):
    address: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetailsBillingDetailsAddress"
    ]
    """
    Billing address.
    """
    email: NotRequired[str]
    """
    Email address.
    """
    name: NotRequired[str]
    """
    Full name.
    """
    phone: NotRequired[str]
    """
    Billing phone number (including extension).
    """


class PaymentEvaluationCreateParamsPaymentDetailsPaymentMethodDetailsBillingDetailsAddress(
    TypedDict,
):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class PaymentEvaluationCreateParamsPaymentDetailsShippingDetails(TypedDict):
    address: NotRequired[
        "PaymentEvaluationCreateParamsPaymentDetailsShippingDetailsAddress"
    ]
    """
    Shipping address.
    """
    name: NotRequired[str]
    """
    Shipping name.
    """
    phone: NotRequired[str]
    """
    Shipping phone number.
    """


class PaymentEvaluationCreateParamsPaymentDetailsShippingDetailsAddress(
    TypedDict,
):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class InvoicePayParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    forgive: NotRequired[bool]
    """
    In cases where the source used to pay the invoice has insufficient funds, passing `forgive=true` controls whether a charge should be attempted for the full amount available on the source, up to the amount to fully pay the invoice. This effectively forgives the difference between the amount available on the source and the amount due.

    Passing `forgive=false` will fail the charge if the source hasn't been pre-funded with the right amount. An example for this case is with ACH Credit Transfers and wires: if the amount wired is less than the amount due by a small amount, you might want to forgive the difference. Defaults to `false`.
    """
    mandate: NotRequired["Literal['']|str"]
    """
    ID of the mandate to be used for this invoice. It must correspond to the payment method used to pay the invoice, including the payment_method param or the invoice's default_payment_method or default_source, if set.
    """
    off_session: NotRequired[bool]
    """
    Indicates if a customer is on or off-session while an invoice payment is attempted. Defaults to `true` (off-session).
    """
    paid_out_of_band: NotRequired[bool]
    """
    Boolean representing whether an invoice is paid outside of Stripe. This will result in no charge being made. Defaults to `false`.
    """
    payment_method: NotRequired[str]
    """
    A PaymentMethod to be charged. The PaymentMethod must be the ID of a PaymentMethod belonging to the customer associated with the invoice being paid.
    """
    source: NotRequired[str]
    """
    A payment source to be charged. The source must be the ID of a source belonging to the customer associated with the invoice being paid.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class PaymentAttemptRecordReportGuaranteedParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    guaranteed_at: NotRequired[int]
    """
    When the reported payment was guaranteed. Measured in seconds since the Unix epoch.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    payment_evaluations: NotRequired[List[str]]
    """
    Payment evaluations associated with this reported payment.
    """
    payment_method_details: NotRequired[
        "PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetails"
    ]
    """
    Information about the Payment Method debited for this payment.
    """
    processor_details: NotRequired[
        "PaymentAttemptRecordReportGuaranteedParamsProcessorDetails"
    ]
    """
    Processor information for this payment.
    """


class PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetails(
    TypedDict
):
    card: NotRequired[
        "PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetailsCard"
    ]
    """
    Information about the card payment method used to make this payment.
    """
    type: Literal["card"]
    """
    The type of the payment method details. An additional hash is included on the payment_method_details with a name matching this value. It contains additional information specific to the type.
    """


class PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetailsCard(
    TypedDict,
):
    checks: NotRequired[
        "PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetailsCardChecks"
    ]
    """
    Verification checks performed on the card.
    """


class PaymentAttemptRecordReportGuaranteedParamsPaymentMethodDetailsCardChecks(
    TypedDict,
):
    address_line1_check: NotRequired[
        Literal["fail", "pass", "unavailable", "unchecked"]
    ]
    """
    The result of the check on the cardholder's address line 1.
    """
    address_postal_code_check: NotRequired[
        Literal["fail", "pass", "unavailable", "unchecked"]
    ]
    """
    The result of the check on the cardholder's postal code.
    """
    cvc_check: NotRequired[Literal["fail", "pass", "unavailable", "unchecked"]]
    """
    The result of the check on the card's CVC.
    """


class PaymentAttemptRecordReportGuaranteedParamsProcessorDetails(TypedDict):
    custom: NotRequired[
        "PaymentAttemptRecordReportGuaranteedParamsProcessorDetailsCustom"
    ]
    """
    Information about the custom processor used to make this payment.
    """
    type: Literal["custom"]
    """
    The type of the processor details. An additional hash is included on processor_details with a name matching this value. It contains additional information specific to the processor.
    """


class PaymentAttemptRecordReportGuaranteedParamsProcessorDetailsCustom(
    TypedDict,
):
    payment_reference: str
    """
    An opaque string for manual reconciliation of this payment, for example a check number or a payment processor ID.
    """
